import os
import tempfile
import mock
import pytest
import yaml
import os_sdk_light as osl


@pytest.fixture
def change_dir():
    dir = os.getcwd()
    os.chdir(os.path.dirname(__file__))
    yield
    os.chdir(dir)


def test_no_cloud_config(change_dir):
    with pytest.raises(osl.exceptions.CannotConnectToCloud, match="Not enough params"):
        osl.get_client(0, 0, 0)


def test_invalid_auth_params():
    f = tempfile.NamedTemporaryFile(mode="w")
    data = yaml.safe_load(open("clouds.yaml"))
    data["clouds"]["devstack"]["auth"]["password"] = "fake"
    yaml.dump(data, f)
    with pytest.raises(
        osl.exceptions.CannotConnectToCloud, match="Authentication failed"
    ):
        osl.get_client("devstack", 0, 0, config={"config_files": [f.name]})


def test_incorrect_auth_schema():
    f = tempfile.NamedTemporaryFile(mode="w")
    data = yaml.safe_load(open("clouds.yaml"))
    del data["clouds"]["devstack"]["auth"]["password"]
    yaml.dump(data, f)
    with pytest.raises(
        osl.exceptions.CannotConnectToCloud, match="Authentication failed"
    ):
        osl.get_client("devstack", 0, 0, config={"config_files": [f.name]})


def test_empty_auth_schema():
    f = tempfile.NamedTemporaryFile(mode="w")
    data = yaml.safe_load(open("clouds.yaml"))
    del data["clouds"]["devstack"]["auth"]["password"]
    del data["clouds"]["devstack"]["auth"]["username"]
    yaml.dump(data, f)
    with pytest.raises(
        osl.exceptions.CannotConnectToCloud, match="Authentication failed"
    ):
        osl.get_client("devstack", 0, 0, config={"config_files": [f.name]})


def test_get_endpoint_failed():
    with pytest.raises(
        osl.exceptions.CannotConnectToCloud, match="Failed to find service endpoint"
    ):
        osl.get_client("devstack", 0, 0)


def test_incorrect_schema():
    f = tempfile.NamedTemporaryFile()
    with pytest.raises(osl.exceptions.SchemaError, match="cannot be read or invalid"):
        osl.get_client("devstack", "compute", f.name)


def test_request_not_match_schema():
    compute = osl.get_client("devstack", "compute", osl.schema("compute.yaml"))
    with pytest.raises(osl.exceptions.ValidationError, match="required parameter"):
        compute.flavors.create_flavor()
    with pytest.raises(osl.exceptions.ValidationError, match="required property"):
        compute.flavors.create_flavor(flavor={"flavor": {"name": "test"}})


def test_response_not_match_schema():
    compute = osl.get_client("devstack", "compute", osl.schema("compute.yaml"))
    with mock.patch("bravado.http_future.HttpFuture._get_incoming_response") as r:
        resp = mock.MagicMock()
        resp.status_code = 200
        resp.json.return_value = {}
        r.return_value = resp
        with pytest.raises(osl.exceptions.ValidationError, match="required property"):
            compute.flavors.get_flavor(flavor_id="fake")


def test_not_defined_response():
    compute = osl.get_client("devstack", "compute", osl.schema("compute.yaml"))
    with mock.patch("bravado.requests_client.RequestsResponseAdapter") as resp_class:
        resp = mock.MagicMock()
        resp.status_code = 404
        resp.json.return_value = {}
        resp_class.return_value = resp
        try:
            compute.flavors.get_flavor(flavor_id="fake")
            pytest.fail("Exception must be thrown")
        except osl.exceptions.UnexpectedResponse as e:
            assert resp.status_code == e.status_code


def test_microversion_in_header():
    p = osl.get_client("devstack", "placement", osl.schema("placement.yaml"))
    p.traits.get_traits()



import pytest
import uuid
import os_sdk_light as osl
from bravado_core import exception


TEST_RP = "test resource provider %s" % uuid.uuid4()
CLOUD = "devstack"
c = osl.get_client(CLOUD, "placement", osl.schema("placement.yaml"))


@pytest.fixture
def rp():
    orig_rp = {"name": TEST_RP}
    c.resource_providers.create_resource_provider(body=orig_rp)
    rp = c.resource_providers.list_resource_providers(
        name=orig_rp["name"]
    )["resource_providers"][0]
    yield rp
    c.resource_providers.delete_resource_provider(uuid=rp["uuid"])


def test_rps(rp):
    rp1 = c.resource_providers.get_resource_provider(uuid=rp["uuid"])
    assert rp["name"] == rp1["name"]


def test_inventories(rp):
    c.resource_providers.put_inventories(
        uuid=rp["uuid"],
        body={
            "resource_provider_generation": rp["generation"],
            "inventories": {"MEMORY_MB": {"total": 512}},
        },
    )
    invs = c.resource_providers.get_inventories(uuid=rp["uuid"])
    c.resource_providers.put_inventories(
        uuid=rp["uuid"],
        body={
            "resource_provider_generation": invs["resource_provider_generation"],
            "inventories": {},
        },
    )


def test_aggregates(rp):
    u1 = "%s" % uuid.uuid4()
    u2 = "%s" % uuid.uuid4()
    c.resource_providers.put_aggregates(uuid=rp["uuid"], body=[u1, u2])
    aggs = c.resource_providers.get_aggregates(uuid=rp["uuid"])[
        "aggregates"
    ]
    assert set([u1, u2]) == set(aggs)
    c.resource_providers.put_aggregates(uuid=rp["uuid"], body=[])


def test_traits():
    name = f'custom-trait-{uuid.uuid4()}'.replace('-', '_').upper()
    c.traits.set_trait(name=name)
    c.traits.get_trait(name=name)
    c.traits.delete_trait(name=name)



import time
import pytest
import uuid
import os_sdk_light as osl
from bravado_core import exception

TEST_FLAVOR = "test flavor %s" % uuid.uuid4()
TEST_SERVER = "test server %s" % uuid.uuid4()
CLOUD = "devstack"
compute_client = osl.get_client(CLOUD, "compute", osl.schema("compute.yaml"))
image_client = osl.get_client(CLOUD, "image", osl.schema("image.yaml"))
network_client = osl.get_client(CLOUD, "network", osl.schema("network.yaml"))


@pytest.fixture
def server():
    flavors = compute_client.flavors.list_flavors()["flavors"]
    flavors.sort(key=lambda x: x["ram"])
    flavor = flavors[0]
    images = image_client.images.list()["images"]
    image = [i for i in images if "cirros" in i["name"]][0]
    networks = network_client.networks.list()["networks"]
    network = [i for i in networks if "private" in i["name"]][0]
    server = compute_client.servers.create_server(
        server={
            "server": {
                "name": TEST_SERVER,
                "imageRef": image["id"],
                "flavorRef": flavor["id"],
                "networks": [{"uuid": network["id"]}],
            }
        }
    )["server"]
    for i in range(20):
        server = compute_client.servers.get_server(server_id=server['id'])['server']
        if server['status'] == 'ACTIVE':
            break
        time.sleep(1)
    else:
        raise ValueError('Timeout waiting server active')
    yield server
    compute_client.servers.delete_server(server_id=server["id"])



import pytest
import uuid
import os_sdk_light as osl
from bravado_core import exception


TEST_FLAVOR = "test flavor %s" % uuid.uuid4()
TEST_SERVER = "test server %s" % uuid.uuid4()
CLOUD = "devstack"
compute_client = osl.get_client(CLOUD, "compute", osl.schema("compute.yaml"))
image_client = osl.get_client(CLOUD, "image", osl.schema("image.yaml"))
network_client = osl.get_client(CLOUD, "network", osl.schema("network.yaml"))


def test_list_flavors():
    for flavor in compute_client.flavors.list_flavors()["flavors"]:
        assert "id" in flavor
        assert "name" in flavor


def test_create_flavor():
    with pytest.raises(osl.exceptions.ValidationError):
        compute_client.flavors.create_flavor()
    flavor = compute_client.flavors.create_flavor(
        flavor={"flavor": {"name": TEST_FLAVOR, "ram": 16384, "disk": 1, "vcpus": 2}}
    )["flavor"]
    new_flavor = compute_client.flavors.get_flavor(flavor_id=flavor["id"])["flavor"]
    assert flavor["name"] == new_flavor["name"]
    compute_client.flavors.delete_flavor(flavor_id=flavor["id"])


def test_server():
    flavors = compute_client.flavors.list_flavors()["flavors"]
    flavors.sort(key=lambda x: x["ram"])
    flavor = flavors[0]
    images = image_client.images.list()["images"]
    image = [i for i in images if "cirros" in i["name"]][0]
    networks = network_client.networks.list()["networks"]
    network = [i for i in networks if "private" in i["name"]][0]
    server = compute_client.servers.create_server(
        server={
            "server": {
                "name": TEST_SERVER,
                "imageRef": image["id"],
                "flavorRef": flavor["id"],
                "networks": [{"uuid": network["id"]}],
            }
        }
    )["server"]
    compute_client.servers.delete_server(server_id=server["id"])



import sys
import six
import os
import logging
import yaml
import keystoneauth1.exceptions as ka_excs
import jsonschema.exceptions
import os_client_config

from six.moves.urllib import parse as urlparse
from bravado.client import SwaggerClient, CallableOperation, ResourceDecorator
from bravado.requests_client import RequestsClient
from swagger_spec_validator.common import SwaggerValidationError
from bravado_core.exception import SwaggerMappingError
from os_sdk_light import exceptions


LOG = logging.getLogger(__name__)
SCHEMAS = os.path.dirname(os.path.realpath(__file__)) + '/schemas/'


class OSLCallableOperation(CallableOperation):
    def __call__(self, **op_kwargs):
        try:
            return super(OSLCallableOperation, self).__call__(
                **op_kwargs).response().result
        except (SwaggerMappingError,
                jsonschema.exceptions.ValidationError) as e:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            LOG.exception(
                'Exception happens during request or response parsing. '
                'Please check schema is valid and server version corresponds '
                'to the declared')
            raise six.reraise(
                exceptions.ValidationError,
                exceptions.ValidationError(e),
                exc_traceback)
        except Exception as e:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            LOG.exception(
                'Exception happens while getting response from server. '
                'This could be part of the normal flow '
                'like 404 for non existing objects')
            new_exception = exceptions.UnexpectedResponse(e, e)
            raise six.reraise(
                exceptions.UnexpectedResponse,
                new_exception,
                exc_traceback)


class OSLResourceDecorator(ResourceDecorator):
    def __getattr__(self, name):
        """
        :rtype: :class:`CallableOperation`
        """
        return OSLCallableOperation(getattr(self.resource, name), False)


class OSLSwaggerClient(SwaggerClient):
    def _get_resource(self, item):
        """
        :param item: name of the resource to return
        :return: :class:`Resource`
        """
        resource = self.swagger_spec.resources.get(item)
        if not resource:
            raise AttributeError(
                'Resource {0} not found. Available resources: {1}'
                .format(item, ', '.join(dir(self))))

        # Wrap bravado-core's Resource and Operation objects in order to
        # execute a service call via the http_client.
        return OSLResourceDecorator(resource, False)


class OSLRequestsClient(RequestsClient):
    def request(self, *args, **kwargs):
        http_future = super(OSLRequestsClient, self).request(*args, **kwargs)
        if hasattr(self, 'custom_headers'):
            for header, value in self.custom_headers:
                http_future.future.request.headers.setdefault(header, value)
        return http_future


def schema(name):
    return SCHEMAS + name


def get_client(cloud, service, schema, config={}, cloud_config={}):
    try:
        if cloud_config:
            cnf = os_client_config.OpenStackConfig()
            cnf.cloud_config = cloud_config
            cloud = cnf.get_one(cloud)
        else:
            cloud = os_client_config.OpenStackConfig(**config).get_one(cloud)
    except ka_excs.MissingRequiredOptions as e:
        LOG.exception(
            'Not enough params to build a cloud connection. '
            'Please provide config file or environment variables, '
            'See https://docs.openstack.org/'
            'os-client-config/latest/user/configuration.html')
        raise exceptions.CannotConnectToCloud(
            'Not enough params to build a cloud connection: %s' % e)

    adapter = cloud.get_session_client(service)
    try:
        access_info = adapter.session.auth.get_access(adapter.session)
    except (ka_excs.Unauthorized, ka_excs.BadRequest) as e:
        LOG.exception(
            'Cloud authentication failed. '
            'Please check credintial and auth_url')
        raise exceptions.CannotConnectToCloud('Authentication failed: %s' % e)
    endpoints = access_info.service_catalog.get_endpoints()
    try:
        interface = cloud.config.get('interface', 'public')
        endpoint = [e for e in endpoints[service]
                    if e['interface'] == interface][0]
    except (KeyError, IndexError, TypeError) as e:
        LOG.exception(
            'Endpoint for service %s with interface %s is not found. '
            'Try to check service exists in service catalog.'
            % (service, interface))
        raise exceptions.CannotConnectToCloud(
            'Failed to find service endpoint: %s' % e)
    try:
        with open(schema) as f:
            spec = yaml.safe_load(f)
            SwaggerClient.from_spec(spec)
    except (IOError, ValueError, SwaggerValidationError) as e:
        LOG.exception(
            'Schema file %s cannot be read or incorrect. '
            'Please check file exists, accessible '
            'and satisfies swagger 2.0 specification.' % schema)
        raise exceptions.SchemaError(
            'Schema file cannot be read or invalid: %s' % e)

    client_endpoint = endpoint['url'].rstrip('/') + spec['basePath']
    url = urlparse.urlsplit(endpoint['url'])
    spec['host'] = url.netloc
    spec['basePath'] = url.path.rstrip('/') + spec['basePath']
    spec['schemes'] = [url.scheme]
    LOG.debug('Got swagger server configuration for service %s: %s%s',
              service, spec['host'], spec['basePath'])

    http_client = OSLRequestsClient()
    http_client.set_api_key(
        url.netloc, access_info.auth_token,
        param_name='x-auth-token', param_in='header'
    )
    version_header = spec['info'].get('x-version-header')
    if version_header:
        template = spec['info'].get('x-version-header-value-template', '%s')
        http_client.custom_headers = [
            (version_header, template % spec['info']['version'])]

    client = OSLSwaggerClient.from_spec(
        spec,
        http_client=http_client,
        config=config)
    client.endpoint = client_endpoint
    client.token = access_info.auth_token
    return client



import pytest
import uuid
import os_sdk_light as osl
from bravado_core import exception

CLOUD = "devstack"
c = osl.get_client(cloud=CLOUD, service="image", schema=osl.schema("image.yaml"))


def test_image():
    image = c.images.create_image(image={'name': 'myubuntu'})
    image = c.images.get_image(image_id=image['id'])
    # update = [{"op": "replace", "path": "/name", "value": image["id"]}]
    # image = c.images.update_image(image_id=image['id'], image=update)
    c.images.delete_image(image_id=image['id'])



class BaseException(Exception):
    pass


class CannotConnectToCloud(BaseException):

    """Error while gettting endpoint for service.

    Errors like invalid connection configuration, invalid auth,
    no endpoint exists, etc.

    """

    pass


class SchemaError(BaseException):

    """Schema reading related error.

    Errors like no schema file, invalid format, etc.

    """

    pass


class ValidationError(BaseException):

    """Error when input/output does not match correcpoding schema.

    Both request and response could have json schemas, if they does not match
    actual data the exception occured.

    """

    pass


class UnexpectedResponse(BaseException):

    """This happens if client does not get proper response.

    For example, a server returns not expected status like 404, 500.
    Client cannot create session or connection timeouted.

    """

    def __init__(self, message, orig_exception):
        super(UnexpectedResponse, self).__init__(message)
        self.orig_exception = orig_exception

    @property
    def status_code(self):
        if hasattr(self.orig_exception, 'status_code'):
            return self.orig_exception.status_code
        return -1

    @property
    def message(self):
        if hasattr(self.orig_exception, 'message'):
            return self.orig_exception.message
        return ''



#!/usr/bin/env python

from setuptools import setup, find_packages
setup(
    name="os-sdk-light",
    version="0.2.5",
    packages=find_packages(),
    # metadata for upload to PyPI
    author="Andrey Volkov",
    author_email="amadev@mail.ru",
    description="Lightweight OpenStack client with schema based approach",
    license="MIT",
    keywords="openstack client swagger openapi",
    url="https://github.com/amadev/os-sdk-light",
    install_requires=[
        'os-client-config',
        'bravado',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',
    ],
    include_package_data=True,
)



import pytest
import uuid
import os_sdk_light as osl
from bravado_core import exception

CLOUD = "devstack"
c = osl.get_client(cloud=CLOUD, service="network", schema=osl.schema("network.yaml"))


def test_port():
    networks = c.networks.list()["networks"]
    network = [i for i in networks if "private" in i["name"]][0]
    port_id = str(uuid.uuid4())
    port = c.ports.create_port(port={"port": {"network_id": network["id"]}})["port"]
    port = c.ports.get_port(port_id=port["id"])["port"]
    c.ports.list_ports(id=port["id"])
    port = c.ports.update_port(port_id=port["id"], port={"port": {"name": port_id}})[
        "port"
    ]
    c.ports.delete_port(port_id=port["id"])


def test_floatingip(server):
    networks = c.networks.list()["networks"]
    network = [i for i in networks if "public" in i["name"]][0]
    port = c.ports.list_ports(device_id=server["id"])['ports'][0]
    floatingip_id = str(uuid.uuid4())
    floatingip = c.floatingips.create_floatingip(
        floatingip={
            "floatingip": {"floating_network_id": network["id"], "port_id": port['id']}
        }
    )["floatingip"]
    floatingip = c.floatingips.get_floatingip(floatingip_id=floatingip["id"])[
        "floatingip"
    ]
    c.floatingips.list_floatingips(id=floatingip["id"])
    floatingip = c.floatingips.update_floatingip(
        floatingip_id=floatingip["id"],
        floatingip={"floatingip": {"description": floatingip_id}},
    )["floatingip"]
    c.floatingips.delete_floatingip(floatingip_id=floatingip["id"])
